﻿using System;
using System.IO;
using System.Linq;
using UnityEditor;
using UnityEditor.Android;
using UnityEngine;

namespace fybase
{
    public class AndroidConfig : IPostGenerateGradleAndroidProject
    {
        public int callbackOrder => 0;
        //     Implement this function to receive a callback after the Android Gradle project
        //     is generated and before building begins. It is not called when doing an Internal
        //     build.
        public void OnPostGenerateGradleAndroidProject(string path)
        {
            string filePath = Path.Combine(path, "build.gradle");
            string[] insertLines = new string[] {
            "    implementation 'com.android.support:appcompat-v7:28.0.0'",
            "    implementation 'com.android.support:cardview-v7:28.0.0'"
            };
            if (!File.Exists(filePath))
            {
                Debug.LogError("FYINFO: 当前项目未导出Gradle项目，无法构建需要的依赖");
                return;
            }
            string[] ls = File.ReadAllLines(filePath);
            int insertIndex = 0;
            foreach (string l in ls)
            {
                ++insertIndex;
                if (l.Contains("implementation fileTree"))
                {
                    break;
                }
            }
            if (insertIndex == 0)
            {
                string tps = string.Join(string.Empty, insertLines.Select(s => s + "\r\n"));
                Debug.LogError($"FYINFO: 当前项目无法找到依赖添加位置，请手动导出的AndroidStudio项目，并在{path}/build.gradle的dependencies字段下添加此属性：\r\n {tps}");
                Debug.LogWarning($"FYINFO: 如果您的Unity项目使用了自定义 Gradle 构建模板【mainTemplate.gradle 】，则可以将\r\n {tps} 添加到对应dependencies属性下且禁用此脚本功能，防止重复添加依赖");
                return;
            }
            string[] resLines = new string[ls.Length + insertLines.Length];
            Array.Copy(ls, 0, resLines, 0, insertIndex);
            Array.Copy(ls, insertIndex, resLines, insertIndex + insertLines.Length, ls.Length - insertIndex);
            foreach (string il in insertLines)
            {
                resLines[insertIndex++] = il;
            }
            File.WriteAllLines(filePath, resLines);
            //Debug.Log($"如果出现Gradle构建失败，请修改{path}/build.gradle文件编码为UTF-8非BOM编码");
            //Debug.Log($"如果出现Gradle构建失败，请查看{path}/build.gradle依赖是否重复");
        }
    }
}
