﻿using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using UnityEditor;
using UnityEngine;
namespace fybase
{
    [InitializeOnLoad]
    public class FYImportTip
    {
        static FYImportTip()
        {
            AssetDatabase.importPackageCompleted += (pn) =>
            {
                if (pn == "FYBase") 
                {
                    // 删除旧版防沉迷SDK Android目录
                    DeleteFolder("Assets/Plugins/FYAntiAddiction/Android");
                    // 删除旧版防沉迷SDK iOS目录
                    DeleteFolder("Assets/Plugins/FYAntiAddiction/iOS");
                }

                // 导入FY开头的包提示一下
                if (pn.StartsWith("FY")) 
                {
                    ShowAlert(pn);
                }
            };
        }

        private static void ShowAlert(string pn)
        {
            Debug.LogWarning($"FYINFO: 检测到您工程中导入了{pn}.unitypackage,请在最后一个星云SDK相关的unitypackage导入完毕后，{ConfigWindow.HelpInfo}");
            ConfigWindow.ShowWindow();
        }

        private static void DeleteFolder(string folderPath)
        {
            if (!Directory.Exists(folderPath))
            {
                Debug.Log("###目录不存在无需删除: " + folderPath);
                return;
            }

            if (DeleteAssetsInFolder(folderPath))
            {
                AssetDatabase.Refresh();
                Debug.Log("###目录删除成功: " + folderPath);
            }
            else
            {
                Debug.Log("###目录删除失败: " + folderPath);
            }
        }

        private static bool DeleteAssetsInFolder(string folderPath)
        {
            bool success = true;
            string[] assets = AssetDatabase.FindAssets("", new string[] { folderPath });
            foreach (string asset in assets)
            {
                string assetPath = AssetDatabase.GUIDToAssetPath(asset);
                // 使用AssetDatabase.DeleteAsset尝试删除资源
                if (!AssetDatabase.DeleteAsset(assetPath))
                {
                    Debug.Log("###文件删除失败: " + assetPath);
                    success = false;
                }
            }
            return success;
        }
    }
}
