﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using System.Xml;
using UnityEditor.Build;
using UnityEditor.Build.Reporting;
using UnityEngine;
namespace fybase
{
    using static ConfigWindow;
    public class PreBuildConfig : IPreprocessBuildWithReport
    {
        public int callbackOrder => 1;
        public void OnPreprocessBuild(BuildReport report)
        {
            InitConfig(out int companyId, out int appID, out string appKey, out bool debug, out GameOrientation orientation,out IssueArea gameIssue);
            if (appID == ORI_APPID || appID == 123456 || ORI_APP_KEY == appKey || appKey.Trim() == "" || appKey.Trim() == "your_app_key")
            {
                throw new BuildFailedException($"FYINFO: 检测到您未进行SDK配置,{HelpInfo}");
            }
            FYCfgInfo cfg = Config(companyId.ToString(),appID.ToString(), appKey, debug, orientation, gameIssue);
            //校验源文件与目标文件 检测是否已经同步可以达到打包要求
            switch (report.summary.platform)
            {
                case UnityEditor.BuildTarget.Android:
                    if (File.Exists(desXMLPath))
                    {
                        List<string> tips = new List<string>();
                        MergeXml(cfg.xml, tips);
                        if (tips.Count > 0)
                            Debug.LogWarning($"FYINFO: {desXMLPath}配置与标准不一致，{HelpInfo}");
                    }
                    else
                    {
                        Error(desXMLPath);
                    }
                    if (!Directory.Exists(desJavaPath))
                        Error("");
                    Compare($"{desJavaPath}/FYSDKActivity.java", cfg.javaActivity);
                    Compare($"{desJavaPath}/FYSDKApplication.java", cfg.javaAppC);
                    Compare($"{desJavaPath}/FYSDKBridge.java", cfg.javaBridge);
                    break;
                case UnityEditor.BuildTarget.iOS:
                    Compare(FYCfgInfo.XcodeConfigPath, new Regex(@"\\u([0-9A-F]{4})", RegexOptions.IgnoreCase | RegexOptions.Compiled)
                        .Replace(cfg.xcodeConfig, x => string.Empty + Convert.ToChar(Convert.ToUInt16(x.Result("$1"), 16))));
                    Compare(desPlistPath, cfg.plist);
                    if (!Directory.Exists(desObjcPath))
                        Error("");
                    Compare($"{desObjcPath}/FYSDKAppController.mm", cfg.objcAppC);
                    Compare($"{desObjcPath}/FYSDKBridge.mm",cfg.objcBridge);
                    break;
                default: break;
            }
        }
        private void Compare(string filePath,string refer) {
            if (File.Exists(filePath))
            {
                //if (!File.ReadAllText(filePath).Trim().Equals(refer.Trim()))
                if (File.ReadAllText(filePath).Trim().Length != refer.Trim().Length)
                    Warning(filePath);//一般可忽略此消息，只是警告
            }
            else
            {
                Error(filePath);
            }
        }
    }
}
